unit option;

interface

uses
  //personal units :
  language,
  //generic units :
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, IniFiles;

type
  TFormOptions = class(TForm)
    Label2: TLabel;
    txt_demuxExe: TEdit;
    Label1: TLabel;
    Label3: TLabel;
    txt_muxExe: TEdit;
    Label4: TLabel;
    Label5: TLabel;
    txt_authorExe: TEdit;
    Label6: TLabel;
    btn_ok: TButton;
    btn_cancel: TButton;
    procedure btn_cancelClick(Sender: TObject);
    procedure btn_okClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Dclarations prives }
    Ini: TMemIniFile; //the ini file where options are saved
    function checkEXEpresent(): boolean; //check if program file is present, otherwise display an error message

  public
    { Dclarations publiques }
    constructor Create(AOwner: TComponent);
    procedure readIniFile(); //load settings from the ini file and update global variables
  end;

var
  FormOptions: TFormOptions;

implementation

{$R *.dfm}
uses main;

//--------------------------------------------------------------------------------------------------------
constructor TFormOptions.Create(AOwner: TComponent);
{}
begin
    Inherited create(AOwner);
    Ini := TMemIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));//the ini file will be in the same location as the executable

    readIniFile; //load settings from ini file

    //update control on the form from current settings :
    txt_demuxExe.text := optDmuxExePath;
    txt_muxExe.Text := optMuxExePath;
    txt_authorExe.Text := optAuthorExePath;

end;

//--------------------------------------------------------------------------------------------------------
procedure TFormOptions.readIniFile();
begin
    with Ini do begin
        optDmuxExePath := ReadString('software path', 'demuxing', 'C:\Program Files\bbdmux.exe');
        optMuxExePath := ReadString('software path', 'muxing', 'C:\Program Files\DVDStyler\dvdauthor\mplex.exe');
        optAuthorExePath := ReadString('software path', 'authoring', 'C:\Program Files\DVDStyler\dvdauthor\dvdauthor.exe');
    end;

    checkEXEpresent;
end;

//--------------------------------------------------------------------------------------------------------
function TFormOptions.checkEXEpresent(): boolean;
{check that the path to program file is correctly defined}
begin
    if FileExists(optDmuxExePath) = false then begin
        MessageDlg(optDmuxExePath + rstrEXE_IS_MISSING, mtError, [mbOK], 0);
        result := false;
        exit;
    end;

    if FileExists(optMuxExePath) = false then begin
        MessageDlg(optMuxExePath + rstrEXE_IS_MISSING, mtError, [mbOK], 0);
        result := false;
        exit;
    end;

    if FileExists(optAuthorExePath) = false then begin
        MessageDlg(optAuthorExePath + rstrEXE_IS_MISSING, mtError, [mbOK], 0);
        result := false;
        exit;
    end;

    result := true;
end;


//--------------------------------------------------------------------------------------------------------
procedure TFormOptions.btn_okClick(Sender: TObject);
begin

        //store settings in global variables :
        optDmuxExePath := txt_demuxExe.Text;
        optMuxExePath := txt_muxExe.Text;
        optAuthorExePath := txt_authorExe.Text;
        if checkEXEpresent = false then begin
            exit;
        end;

        //Store settings inside the ini file :
        with Ini do begin
            WriteString('software path', 'demuxing', optDmuxExePath);
            WriteString('software path', 'muxing', optMuxExePath);
            WriteString('software path', 'authoring', optAuthorExePath);

            UpdateFile; //save settings into the ini file
        end;

        //close it :
        ModalResult := mrOk;
end;

//--------------------------------------------------------------------------------------------------------
procedure TFormOptions.btn_cancelClick(Sender: TObject);
begin
    ModalResult := mrCancel;
end;

//--------------------------------------------------------------------------------------------------------
procedure TFormOptions.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    Ini.Free; //destroy the connection to the ini file
end;


end.
