unit taskDialog;

{
______________________________________________________________________________
    manage the dialog that will be used to define accuratly
    every task characteristics 
______________________________________________________________________________
}
{TurboVOB - A freeware to prepare DTH7000's video for DVD burning on computer under windows.
Copyright (C) 2005 Didier CADIOU

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program (see gpl.txt file); if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.}
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, task, language, BrowseDialog, ExtCtrls, StrUtils,
  ComCtrls;

type
  TFormTaskManager = class(TForm)
    btnOK: TButton;
    btnCancel: TButton;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    Panel_demux: TPanel;
    Label4: TLabel;
    txt_origVOBfilePath: TEdit;
    chb_deleteOrigVob: TCheckBox;
    btn_BrowseForOriginalVOB: TButton;
    Panel_streams: TPanel;
    Label1: TLabel;
    txt_m2vFilePath: TEdit;
    chb_deleteStreams: TCheckBox;
    btn_BrowseForVideoStream: TButton;
    btn_BrowseForAudioStream: TButton;
    Panel_author: TPanel;
    Label3: TLabel;
    txt_VOBFilePath: TEdit;
    Label5: TLabel;
    txt_DVDfolderPath: TEdit;
    chb_deleteVob: TCheckBox;
    btn_BrowseForDVD: TButton;
    btn_BrowseForVob: TButton;
    chb_author: TCheckBox;
    chb_Demux: TCheckBox;
    Panel_chapter: TPanel;
    Label2: TLabel;
    btn_ImportChapter: TButton;
    edt_chapterList: TRichEdit;
    txt_ac3FilePath: TEdit; //dialog to choose a destination folder
    procedure btnCancelClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure chb_DemuxClick(Sender: TObject);
    procedure chb_authorClick(Sender: TObject);
    procedure disableUselessControls();
    procedure btn_BrowseForVideoStreamClick(Sender: TObject);
    procedure btn_BrowseForAudioStreamClick(Sender: TObject);
    procedure btn_BrowseForDVDClick(Sender: TObject);
    procedure btn_BrowseForVobClick(Sender: TObject);
    procedure btn_BrowseForOriginalVOBClick(Sender: TObject);
    procedure btn_ImportChapterClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure chb_deleteOrigVobClick(Sender: TObject);
    procedure chb_deleteStreamsClick(Sender: TObject);
  private
    { Dclarations prives }
    theTask: Ttask;
    rep: integer; //for user answer
    folderDialog: TBrowseDialog;
    procedure importChapter(chapterFile: TFileName); //to import chapter location from a txt file
    function getChapterList(): string; //to read the chapter list from the form

  public
    { Dclarations publiques }
    constructor Create(AOwner: TComponent; var aTask: Ttask);
    procedure BrowseForOriginalVOB;
  end;

var
  FormTaskManager: TFormTaskManager;

implementation
{$R *.dfm}

//--------------------------------------------------------------------------------------------------------
constructor TFormTaskManager.Create(AOwner: TComponent; var aTask: Ttask);
{argument :
AOwner => parent form ?
newTask => task object (passed by variable) that will be updated by this dialog}
begin
    Inherited create(AOwner);
    theTask := aTask;

    chb_Demux.Checked := aTask.ToBeDemuxed;
    chb_author.Checked := aTask.ToBeAuthored;
    txt_origVOBfilePath.Text := aTask.originalVobPath;
    txt_m2vFilePath.Text := aTask.videoStreamPath;
    txt_ac3FilePath.Text := aTask.audioStreamPath;
    txt_VOBFilePath.Text := aTask.VobPath;
    txt_DVDfolderPath.Text := aTask.destinationFolder;
    chb_deleteOrigVob.Checked := aTask.deleteOriginalVob;
    chb_deleteStreams.Checked := aTask.deleteStreams;
    chb_deleteVob.Checked := aTask.deleteRemuxedVob;

    disableUselessControls;


end;

//--------------------------------------------------------------------------------------------------------
procedure TFormTaskManager.FormShow(Sender: TObject);
{automatically read chapters location if the expected file exists}
begin
    if length(theTask.chapterListStr)>0 then begin //if chapter has been already defined
        edt_chapterList.Text := AnsiReplaceText(theTask.chapterListStr, ',', #13#10); //load it
    end else begin //if chapter aren't currently defined
        {if (FileExists(ExtractFilePath(txt_origVOBfilePath.Text) + 'CellTimes.txt') = true) then begin //if a celltimes.txt file exits
            importChapter(ExtractFilePath(txt_origVOBfilePath.Text) + 'CellTimes.txt'); //load it
        end;}
    end;
end;

//--------------------------------------------------------------------------------------------------------
procedure TFormTaskManager.btnCancelClick(Sender: TObject);
begin
    ModalResult := mrCancel;
end;

//--------------------------------------------------------------------------------------------------------
procedure TFormTaskManager.btnOKClick(Sender: TObject);
begin
    //various checks :
    //potential overwrite of source file
    if txt_origVOBfilePath.text = txt_VOBFilePath.text then begin
        rep := MessageDlg(rstrVOB_FILE_HAS_SAME_NAME, mtWarning, [mbAbort, mbIgnore], 0);
        if rep = mrAbort then exit;
    end;

    //source file missing
    if chb_Demux.Checked = true then begin //demux need a VOB source file
        if FileExists(txt_origVOBfilePath.Text) = false then begin
            MessageDlg(txt_origVOBfilePath.Text + rstrFILE_MISSING, mtWarning, [mbOK], 0);
            exit;
        end;
    end else begin //no demux means that source files are demuxed streams
        if FileExists(txt_m2vFilePath.Text) = false then begin
            MessageDlg(txt_m2vFilePath.Text + rstrFILE_MISSING, mtWarning, [mbOK], 0);
            exit;
        end;
        if FileExists(txt_ac3FilePath.Text) = false then begin
            MessageDlg(txt_ac3FilePath.Text + rstrFILE_MISSING, mtWarning, [mbOK], 0);
            exit;
        end;
    end; //if demux

    //validate choices :
    with theTask do begin
        if chb_Demux.Checked = true then begin
            originalVobPath := txt_origVOBfilePath.Text;
        end else begin //if no demux
            originalVobPath := '';
        end;

        videoStreamPath := txt_m2vFilePath.Text;
        audioStreamPath := txt_ac3FilePath.Text;

        if chb_author.Checked = true then begin
            VobPath := txt_VOBFilePath.Text;
            destinationFolder := txt_DVDfolderPath.Text;
            chapterListStr := getChapterList;
        end else begin //if no remux/authoring
            VobPath := '';
            destinationFolder := '';
            chapterListStr := '';
        end;

        ToBeDemuxed := chb_Demux.Checked;
        ToBeAuthored := chb_author.Checked;
        deleteOriginalVob := chb_deleteOrigVob.Checked;
        deleteStreams := chb_deleteStreams.Checked;
        deleteRemuxedVob := chb_deleteVob.Checked;
    end; //with theTask



    ModalResult := mrOK;
end;
//--------------------------------------------------------------------------------------------------------
function TFormTaskManager.getChapterList(): string;
{convert the chapter list in the form to a string with all values separated by ','}
 var
    ix: integer; //loop index
    OneSec: real; //1 seconde expressed in day
    chapterMark: string; //the chapter mark currently analyzed
 begin
        result := ''; //init
        OneSec := 1/(24*3600);
        for ix := 0 to (edt_chapterList.Lines.Count -1) do begin //sweep through all chapter marks
            chapterMark := edt_chapterList.Lines[ix];

            //convert mark locations if needed (and possible) :
            if LastDelimiter(':',chapterMark) = 0 then begin //no ':' separator => it could means that marks are expressed as frame number
                {try to convert the string as if it is a frame number
                This conversion it is done assuming that video standard is PAL, so 25 frames per sec.
                For NTSC, the conversion is a little bit wrong, but that's it }
                try
                    chapterMark := TimeToStr(((StrToFloat(chapterMark))/25)*OneSec);
                except
                    {if the time conversion fails, an exception will be generated but not catched,
                    therefore the program continue and simply ignore this chapter location}
                end;
            end;//if no ':' separator

            if LastDelimiter(':',chapterMark) = 9 then begin //no ':' separator is expected at position 8, except if time is expressed as hh:mm:ss:msec
                // instead of hh:mm:ss.msec (mpeg2schnitt do that sometime)
                {try to convert the string to replace the last ':' by a '.'}
                chapterMark := leftStr(chapterMark, 8) + '.' + rightStr(chapterMark, length(chapterMark)-9);
            end;



            //Add the current mark to the output string :
            try
                StrToTime(chapterMark); //only to generate a EConvertError if the string is not a valid time
                result := result + chapterMark + ',';
            except //if an error happened while converting the chapter location
                {if the time conversion fails in line above, an exception will be generated but not catched,
                therefore the program continue and simply ignore this corrupted chapter location}
            end;

        end; //sweep through all chapter marks

        result := LeftStr(result, LastDelimiter(',', result)-1);
                //remove all characters at the end of the string,
                //starting after the last ',' delimiter (included)
 end;

//--------------------------------------------------------------------------------------------------------
procedure TFormTaskManager.btn_BrowseForOriginalVOBClick(Sender: TObject);
begin
    BrowseForOriginalVOB;
end;
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
procedure TFormTaskManager.BrowseForOriginalVOB;
begin
    with OpenDialog1 do begin
        if length(txt_origVOBfilePath.Text)>0 then begin
            FileName := txt_origVOBfilePath.Text; //apply previous filename if it exists
        end;
        DefaultExt := 'VOB';
        Filter := 'video file (*.VOB)|*.VOB';
        Title := rstrSELECT_VOB_FILE_SOURCE;
        if Execute then begin
            txt_origVOBfilePath.Text := FileName;
            //autofill the other fields :
            txt_m2vFilePath.Text := ExtractFilePath(filename) + 'VTS_01_1.m2v';
            txt_ac3FilePath.Text := ExtractFilePath(filename) + 'VTS_01_1.ac3';
            txt_VOBFilePath.Text := ExtractFilePath(filename) + 'remuxedVTS_01_1.VOB';
            txt_DVDfolderPath.Text :=
                leftStr(ExtractFilePath(filename), length(ExtractFilePath(filename))-1); //path without the '/'
            if (FileExists(ExtractFilePath(txt_origVOBfilePath.Text) + 'CellTimes.txt') = true) then begin //if a celltimes.txt file exits
                importChapter(ExtractFilePath(txt_origVOBfilePath.Text) + 'CellTimes.txt'); //load it
            end;

        end;
    end;
end;

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
procedure  TFormTaskManager.btn_BrowseForVideoStreamClick(Sender: TObject);
begin
    if chb_Demux.Checked = true then begin //demux will be done, demuxed streams doesn't already exists
        with SaveDialog1 do begin
            FileName := 'VTS_01_1.m2v'; //default filename
            if length(txt_m2vFilePath.Text)>0 then begin
                FileName := txt_m2vFilePath.Text; //apply previous filename if it exists
            end;
            DefaultExt := 'm2v';
            Filter := 'video stream file (*.m2v)|*.m2v';
            Title := rstrSELECT_STREAM_FILE_DESTINATION;
            if Execute then begin
                txt_m2vFilePath.Text := FileName; //select destination
                //autofill the other fields :
                txt_ac3FilePath.Text := ChangeFileExt(FileName, '.ac3'); //automatically update also audio destination
                txt_VOBFilePath.Text := ExtractFilePath(filename) + 'remuxedVTS_01_1.VOB';
                txt_DVDfolderPath.Text :=
                    leftStr(ExtractFilePath(filename), length(ExtractFilePath(filename))-1); //path without the '/'
            end;
        end;
    end else begin //no demux will be done, demuxed streams exists and will be used for authoring
        with OpenDialog1 do begin
            if length(txt_m2vFilePath.Text)>0 then begin
                FileName := txt_m2vFilePath.Text; //apply previous filename if it exists
            end;
            DefaultExt := 'm2v';
            Filter := 'video stream file (*.m2v)|*.m2v';
            Title := rstrSELECT_STREAM_FILE_SOURCE;
            if Execute then begin
                txt_m2vFilePath.Text := FileName; //select source

                //autofill the other fields :
                if FileExists(ChangeFileExt(FileName, '.ac3')) = true then begin//also select audio source if it exists and is named as the video stream
                    txt_ac3FilePath.Text := ChangeFileExt(FileName, '.ac3');
                end else begin //if it doesn't exists as expected
                    txt_ac3FilePath.Text := leftStr(ExtractFilePath(filename), length(ExtractFilePath(filename))-1); //pre-enter only the folder path without the '/' car. (the user will browse manually to find it
                end; //if audio file exits

                txt_VOBFilePath.Text := ExtractFilePath(filename) + 'remuxedVTS_01_1.VOB';
                txt_DVDfolderPath.Text :=
                    leftStr(ExtractFilePath(filename), length(ExtractFilePath(filename))-1); //path without the '/'

                if FileExists(ExtractFilePath(filename) + 'CellTimes.txt') = true then begin
                    importChapter(ExtractFilePath(filename) + 'CellTimes.txt');
                end;
            end; //if execute
        end; //with opendialog1
    end; //chb_Demux.Checked
end;
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
procedure TFormTaskManager.btn_BrowseForAudioStreamClick(Sender: TObject);
begin
    if chb_Demux.Checked = true then begin //demux will be done, demuxed streams doesn't already exists
        with SaveDialog1 do begin
            FileName := 'VTS_01_1.ac3'; //default filename
            if length(txt_ac3FilePath.Text)>0 then begin
                FileName := txt_ac3FilePath.Text; //apply previous filename if it exists
            end;
            DefaultExt := 'ac3';
            Filter := 'audio stream file (*.ac3)|*.ac3';
            Title := rstrSELECT_STREAM_FILE_DESTINATION;
            if Execute then
                txt_ac3FilePath.Text := FileName; //select destination
        end;
    end else begin //no demux will be done, demuxed streams exists and will be used for authoring
        with OpenDialog1 do begin
            if length(txt_ac3FilePath.Text)>0 then begin
                FileName := txt_ac3FilePath.Text; //apply previous filename if it exists
            end;
            DefaultExt := 'ac3';
            Filter := 'audio stream file (*.ac3)|*.ac3';
            Title := rstrSELECT_STREAM_FILE_SOURCE;
            if Execute then
                txt_ac3FilePath.Text := FileName; //select source
        end;
    end;
end;
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
procedure TFormTaskManager.btn_BrowseForVobClick(Sender: TObject);
begin
    with SaveDialog1 do begin
        FileName := 'VTS_01_1.VOB'; //default filename
        if length(txt_VOBFilePath.Text)>0 then begin
            FileName := txt_VOBFilePath.Text; //apply previous filename if it exists
        end;
        DefaultExt := 'VOB';
        Filter := 'video file (*.VOB)|*.VOB';
        Title := rstrSELECT_VOB_FILE_DESTINATION;
        if Execute then begin
            if filename = txt_origVOBfilePath.Text then begin
                rep := MessageDlg(rstrCAUTION_ORIGINAL_AND_DESTINATION_ARE_EQUAL, mtWarning, [mbYes, mbNo], 0);
                if rep = mrNo then exit;
            end;
            txt_VOBFilePath.Text := FileName;
            txt_DVDfolderPath.Text :=
                leftStr(ExtractFilePath(filename), length(ExtractFilePath(filename))-1); //path without the '/'
        end;
    end;
end;
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
procedure TFormTaskManager.btn_BrowseForDVDClick(Sender: TObject);
begin
    folderDialog := TBrowseDialog.Create(FormTaskManager);
    folderDialog.Title := rstrSELECT_DVD_FOLDER_DESTINATION;
    if length(txt_DVDfolderPath.Text)>0 then begin
        folderDialog.Folder := txt_DVDfolderPath.Text; //apply previous folder name if it exists
    end;

    if folderDialog.Execute then //open a specific dialog to ask where is the destination forder
        txt_DVDfolderPath.Text := folderDialog.Folder;

    folderDialog.Free;
end;
//--------------------------------------------------------------------------------------------------------
procedure TFormTaskManager.chb_DemuxClick(Sender: TObject);
begin
    disableUselessControls;
end;

//--------------------------------------------------------------------------------------------------------
procedure TFormTaskManager.chb_authorClick(Sender: TObject);
begin
    disableUselessControls;
end;

//--------------------------------------------------------------------------------------------------------
procedure TFormTaskManager.disableUselessControls();
begin
    //depending on weither demuxing is done or not :
    Panel_demux.Visible := chb_Demux.Checked;
    if chb_Demux.Checked = false then begin //erase original VOB path if demux is not asked
        txt_origVOBfilePath.Text := '';
    end;

    //depending on weither authoring is done or not :
    chb_deleteStreams.Enabled := chb_author.Checked; //streams shouldn't be deleted if no authoring is done afterward
    Panel_chapter.visible := chb_author.Checked; //chapters informations are only usefull for authoring
    Panel_author.Visible := chb_author.Checked;
    if chb_author.Checked = false then begin //erase destination VOB path if authoring is not asked
        txt_VOBFilePath.Text := '';
    end;

    //hide stream panel if nothing is asked :
    Panel_streams.Visible := false;
    if (chb_Demux.Checked = true) or (chb_author.Checked = true) then
        Panel_streams.Visible := true;

end;

//--------------------------------------------------------------------------------------------------------
procedure TFormTaskManager.btn_ImportChapterClick(Sender: TObject);
{Browse to a txt file that contains chapters location
    and import them}
begin
    //open the dialog to choose the file :
    with OpenDialog1 do begin
        FileName := txt_m2vFilePath.Text;
        DefaultExt := 'txt';
        Filter := 'text file (*.txt)|*.txt';
        Title := rstrSELECT_CHAPTER_FILE_SOURCE;
        if Execute then begin
            importChapter(FileName);
        end;
    end;
end;
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
procedure TFormTaskManager.importChapter(chapterFile: TFileName);
{import chapter locations from a file chapterFile to the edit control}
var
    txtFile: textfile; //source file handle
    txt: string; //1 lines from the txt file
    txts: string; //all lines from the txt file, separated by carr. ret. + line feed car
begin
    AssignFile(txtFile, chapterFile); //associate file name to file variable
    reset(txtFile); //open the file

    //read the file to the edit control :
    txts := ''; //init
    while not Eof(txtFile) do begin
        Readln(txtFile, txt);
        txts := txts + txt + #13#10
    end;
    edt_chapterList.Text := txts;
    theTask.chapterListStr := getChapterList ; //affect chapters string to the appropriate task property
        //and benefit of chapter conversion if needed, which is included inside getChapterList fct
    edt_chapterList.Text := AnsiReplaceText(theTask.chapterListStr, ',', #13#10); //replace the chapter display
        //by potentialy converted chapters marks

    CloseFile(txtFile);
end;


//--------------------------------------------------------------------------------------------------------
procedure TFormTaskManager.chb_deleteOrigVobClick(Sender: TObject);
begin
    if chb_deleteOrigVob.Checked = true then begin
        MessageDlg(rstrCAUTION_PT_OF_NO_RETURN, mtWarning, [mbOK], 0);
    end;
end;
//--------------------------------------------------------------------------------------------------------
procedure TFormTaskManager.chb_deleteStreamsClick(Sender: TObject);
begin
    if (chb_deleteStreams.Checked = true) and
        (chb_Demux.checked = false) then begin //if no demux asked, it means that streams are source files
            //and must be preserved otherwise a risk of definitive loss exists
                MessageDlg(rstrCAUTION_PT_OF_NO_RETURN, mtWarning, [mbOK], 0);
    end;
end;

end.
